! PROGRAM AreaSearsHaack
! PURPOSE - Compute the surface of a body of revolution. In particular,
!   solve for the Sears-Haack minimum wave drag (Length-volume) body.
! AUTHOR - Ralph Carmichael, Public Domain Aeronautical Software
! REVISION HISTORY
!   DATE  VERS PERSON  STATEMENT OF CHANGES
! 11Oct02  1.0   RLC   Initial release for the web site

!+
MODULE AreaSearsHaackAux
! ---------------------------------------------------------------------------
IMPLICIT NONE
  INTEGER,PARAMETER,PRIVATE:: WP = SELECTED_REAL_KIND(10,50)

  REAL(WP):: fineness     ! body length / max diameter
  
  REAL(WP),PARAMETER,PRIVATE:: ZERO=0.0_WP, HALF=0.5_WP, ONE=1.0_WP
  REAL(WP),PARAMETER,PRIVATE:: TWO=2.0_WP, THREE=3.0_WP, FOUR=4.0_WP
  REAL(WP),PARAMETER,PUBLIC:: PI = 3.141592653589793238462643_WP
  REAL(WP),PARAMETER,PRIVATE:: TWOPI=TWO*PI
!----------------------------------------------------------------------------

CONTAINS

FUNCTION SearsHaack(x) RESULT(r)
! ---------------------------------------------------------------------------
! PURPOSE - Compute the radius of a Sears-Haack body at a given station
  REAL(WP),INTENT(IN):: x   ! fraction of body length
  REAL(WP):: r   ! radius (as fraction of maximum radius)
!----------------------------------------------------------------------------
  IF ( x <= ZERO .OR. x >= ONE) THEN
    r=ZERO
  ELSE  
    r=(FOUR*x*(ONE-x))**0.75_WP 
  END IF  
  RETURN
END Function SearsHaack   ! -------------------------------------------------
                                                                        
FUNCTION SearsHaackSlope(x) RESULT(drdx)
! ---------------------------------------------------------------------------
! PURPOSE - Compute dr/dx for a Sears-Haack body. This formula is normalized.
! Multiply by radius and divide by total length for the dimensional case).
  REAL(WP),INTENT(IN):: x   ! fraction of body length
  REAL(WP):: drdx   ! dr/dx (normalized)
!----------------------------------------------------------------------------
  IF (x <= ZERO .OR. x >= ONE) THEN 
    drdx=ZERO   ! it is really infinite
  ELSE 
    drdx=THREE*(ONE-x-x)*(FOUR*x*(ONE-x))**(-0.25_WP) 
  END IF 
  RETURN
END Function SearsHaackSlope   ! --------------------------------------------

!+
FUNCTION Fint(x) RESULT(f)
! ---------------------------------------------------------------------------
! PURPOSE - Compute the integrand for use in Quanc8
! NOTE - If you are interested in a different body, the coding changes
!  should be pretty obvious.

  REAL(WP),INTENT(IN):: x   ! integration variable, fraction of body length
  REAL(WP):: f   ! the integrand at x

  REAL(WP):: r,rp
  REAL(WP):: rmax
!----------------------------------------------------------------------------
  rmax=0.5/fineness   ! fineness is a MODULE variable
  r=rmax*SearsHaack(x)
  rp=rmax*SearsHaackSlope(x)
  f=TWOPI*r*Sqrt(ONE+rp**2)
  RETURN
END Function Fint   ! -------------------------------------------------------

END Module AreaSearsHaackAux   ! ============================================

! The following program that uses the module above will reproduce the numbers
! from the table on the Sears-Haack Area page.


!+
PROGRAM AreaSearsHaack
USE AreaSearsHaackAux
IMPLICIT NONE
  INTEGER,PARAMETER:: WP = SELECTED_REAL_KIND(10,50)
  REAL(WP),PARAMETER:: ZERO=0.0, ONE=1.0
!----------------------------------------------------------------------------
  CALL Area()
  STOP

CONTAINS

!+
SUBROUTINE Area()
! ---------------------------------------------------------------------------
USE ForsytheMalcolmMoler,ONLY: Quanc8
  REAL(WP):: a=ZERO, b=ONE
  REAL(WP):: abserr=1E-8, relerr=1E-8
  REAL(WP):: errest
  CHARACTER(LEN=*),PARAMETER:: FMT = '(2I4,F6.3,F9.5, F9.4, ES12.4)'
  REAL(WP):: flag
  INTEGER:: k
  INTEGER:: nofun
  REAL(WP):: result
  REAL(WP):: sCyl   ! area of the side of a cylinder of same length and diam
!----------------------------------------------------------------------------
  WRITE(*,*) 'Surface Area'
  WRITE(*,*) ' n nfun  flag    area   area/sCyl   errest'
  DO k=2,50,2
    fineness=k                   ! fineness is a module variable used by Fint
    sCyl=PI/fineness
    CALL Quanc8 (Fint,a,b,abserr,relerr,result,errest,nofun,flag)
    WRITE(*,FMT) k,nofun,flag,result,result/sCyl,errest
  END DO  
  RETURN
END Subroutine Area   ! -----------------------------------------------------

END Program AreaSearsHaack   ! ==============================================
