!+
MODULE OsAux1
! ---------------------------------------------------------------------------
IMPLICIT NONE
  INTEGER,PRIVATE,PARAMETER:: WP = SELECTED_REAL_KIND(10,50)
  REAL(WP):: machRef,gammaRef   ! set by MaxRampAngle; used by Eq138A
!----------------------------------------------------------------------------
CONTAINS
!+
FUNCTION Eq138A(theta) RESULT(cotDelta)
! ---------------------------------------------------------------------------
! PURPOSE - Special version of Eq138 that picks up mach and gamma as module
!  variables. Used with BrentMin which requires a function of one variable.
  REAL(WP),INTENT(IN):: theta   ! wave angle, radians
  REAL(WP):: cotDelta   ! cotangent of deflection angle.
  
  REAL(WP):: gp1,ms
  REAL(WP),PARAMETER:: ONE=1.0, TWO=2.0
!----------------------------------------------------------------------------
  gp1=gammaRef+ONE                            ! gammaref is a module variable
  ms=(machRef*SIN(theta))**2                   ! machRef is a module variable
  IF (ms <= ONE) THEN
    cotDelta=1E38   ! something really huge 
  ELSE
    cotDelta=TAN(theta)*(gp1*machRef**2/(TWO*(ms-ONE))-ONE)
  END IF
  RETURN
END Function Eq138A   ! -----------------------------------------------------

END Module OsAux1   ! =======================================================

MODULE OsAux2

IMPLICIT NONE
  INTEGER,PRIVATE,PARAMETER:: WP = SELECTED_REAL_KIND(10,50)

CONTAINS
!+
FUNCTION Eq138(mach1,theta,gamma) RESULT(cotDelta)
! ---------------------------------------------------------------------------
! PURPOSE - Equation 138 in NACA Report 1135. Gives the cotangent of the
!  deflection angle for a given wave angle, upstream Mach number and gamma.
! NOTE - Only defined for wave angles greater than the Mach angle. If you
!   should happen to input a value outside this range, you will get a HUGE
!   value returned. If the wave angle is exactly equal to the Mach angle,
!   then the deflection angle is zero and the cotangent is infinite.
  REAL(WP),INTENT(IN):: mach1   ! upstream Mach number
  REAL(WP),INTENT(IN):: theta   ! wave angle, radians
  REAL(WP),INTENT(IN):: gamma   ! ratio of specific heats (usually 1.4)
  REAL(WP):: cotDelta   ! cotangent of deflection angle.
  
  REAL(WP):: gp1,ms
  REAL(WP),PARAMETER:: ONE=1.0, TWO=2.0
!----------------------------------------------------------------------------
  gp1=gamma+ONE
  ms=(mach1*SIN(theta))**2
  IF (ms <= ONE) THEN
    cotDelta=1E38   ! something really huge 
  ELSE
    cotDelta=TAN(theta)*(gp1*mach1**2/(TWO*(ms-ONE))-ONE)
  END IF
  RETURN
END Function Eq138   ! ------------------------------------------------------


!+
FUNCTION OsSlope(theta,mach,gamma) RESULT (deriv)
! ---------------------------------------------------------------------------
! PURPOSE - Oblique Shock. Compute d(delta)/d(theta) for given values of
!  gamma and upstream Mach number.
! NOTE - theta must be in the range of Mach angle to pi/2.
!  Recall, Mach angle = ArcSin(1/mach)
  REAL(WP),INTENT(IN):: theta   ! wave angle, radians
  REAL(WP),INTENT(IN):: mach    ! upstream Mach number
  REAL(WP),INTENT(IN):: gamma   ! ratio of spacific heats 
  REAL(WP):: deriv   ! d(delta)/d(theta)
  
  REAL(WP):: c
  REAL(WP):: cotd
  REAL(WP):: EPS = 1E-12
!  CHARACTER(LEN=*),PARAMETER:: FMT = '(A, 4ES20.12)'
  REAL(WP):: ms,msq
  REAL(WP):: num1
  REAL(WP):: s,t

  REAL(WP):: num,denom
  REAL(WP),PARAMETER:: HALF=0.5, ONE=1.0, TWO=2.0
!----------------------------------------------------------------------------
  c=COS(theta)
  s=SIN(theta)
  t=s/c
  msq=mach*mach
!  ms=msq*SIN(theta)**2-ONE
  ms=msq*s**2 - ONE
  IF (ms <= EPS) THEN
    cotd=Eq138(mach,theta+EPS,gamma)
    c=COS(theta+EPS)
    s=SIN(theta+EPS)
    t=s/c
    ms=msq*s**2 - ONE
    num1=(ONE+t**2)/ms - TWO*msq*s**2/ms**2
    num=HALF*(gamma+ONE)*msq*num1 - ONE/c**2
    denom=-(ONE+cotd**2)
    deriv=num/denom
    RETURN
  END IF
  
  cotd=Eq138(mach,theta,gamma)
  num1=(ONE+TAN(theta)**2)/ms - TWO*msq*SIN(theta)**2/ms**2
  num=HALF*(gamma+ONE)*msq*num1 - ONE/COS(theta)**2
  denom=-(ONE+cotd**2)
  deriv=num/denom

  RETURN
END Function OsSlope   ! ----------------------------------------------------  

!+
SUBROUTINE MaxRampAngle(mach,gamma,maxRamp,thetaMaxRamp)
! ---------------------------------------------------------------------------
! PURPOSE - Compute the maximum deflection for given values of mach and gamma.
!  Include the wave angle where this maximum occurs as an output
! NOTE - Eq138A returns cot(delta). We want a maximum of delta, so we find the
!  minimum of cot(delta)
 USE ForsytheMalcolmMoler,ONLY: BrentMin
 USE OsAux1, ONLY: Eq138A,machRef, gammaRef
  REAL(WP),INTENT(IN):: mach   ! Mach angle
  REAL(WP),INTENT(IN):: gamma  ! ratio of specific heats
  REAL(WP),INTENT(OUT):: maxRamp   ! max deflection angle, radians
  REAL(WP),INTENT(OUT):: thetaMaxRamp  ! theta where max occurs, radians
  
  REAL(WP):: ax,bx
  INTEGER:: errCode
  INTEGER,PARAMETER:: MAXITER=20
  INTEGER:: neval
  REAL(WP),PARAMETER:: TOL=1E-8
  REAL(WP):: xmin,fmin
    REAL(WP),PARAMETER:: HALF=0.5, ONE=1.0
  REAL(WP),PARAMETER:: PI = 3.141592653589793238462643_WP

!----------------------------------------------------------------------------
  machRef=mach
  gammaRef=gamma
  ax=ASIN(ONE/mach)   ! smallest wave angle (Mach angle)
  bx=HALF*PI
  CALL BrentMin(ax,bx,Eq138A,tol,maxIter,neval,errCode,xmin,fmin)
!  IF (errCode /= 0) WRITE(DBG,*) 'Non-zero errCode from BrentMin'
!  WRITE(DBG,*)neval,' iter for mach=',mach
  thetaMaxRamp=xmin
  maxRamp=ATAN(ONE/fmin)
  RETURN
END Subroutine MaxRampAngle   ! ---------------------------------------------

!+
SUBROUTINE ShockAngle(delta,mach,gamma, theta,errCode)
! ---------------------------------------------------------------------------
! PURPOSE - Compute the wave angle (theta) for given values of deflection
!   angle (delta), Mach number and gamma

  REAL(WP),INTENT(IN):: delta
  REAL(WP),INTENT(IN):: mach    ! upstream Mach number
  REAL(WP),INTENT(IN):: gamma
  REAL(WP),INTENT(OUT):: theta
  INTEGER,INTENT(OUT):: errCode
  
  REAL(WP):: deltaMax
  INTEGER:: k
  INTEGER,PARAMETER:: MAX_ITER = 4
  REAL(WP):: r,rp
  REAL(WP):: thetaMaxRamp
  REAL(WP):: w
  REAL(WP),PARAMETER:: ZERO=0.0, ONE=1.0
!----------------------------------------------------------------------------
  IF (mach <= ONE .OR. delta <= ZERO) THEN
    errCode=1   ! invalid input
    theta=ZERO
    RETURN
  END IF  
  
  CALL MaxRampAngle(mach, gamma, deltaMax, thetaMaxRamp)
  IF (delta > deltaMax) THEN
    errCode=2   ! attached shock not possible
    theta=ZERO
    RETURN
  END IF  
  
! If we haven't returned, then there should be a solution  
  w=ASIN(ONE/mach) + 0.00001
  DO k=1,MAX_ITER
    r=ATAN(ONE/Eq138(mach,w,gamma)) ! Eq138 returns cot(delta)
    rp=OsSlope(w,mach,gamma)
    w=w-(r-delta)/rp
  END DO  
  theta=w
  errCode=0   ! success
  RETURN
END Subroutine ShockAngle   ! -----------------------------------------------


END Module OsAux2   ! ========================================================


!+
PROGRAM ObliqueShockStudy
! ---------------------------------------------------------------------------
! PURPOSE - Test subroutine ShockAngle
! AUTHOR  - Ralph L. Carmichael, Public Domain Aeronautical Software
! REVISION HISTORY
!   DATE   VERS PERSON  STATEMENT OF CHANGES
! 09Jun02   0.1   RLC   Original coding
USE OsAux2
IMPLICIT NONE
 
  INTEGER,PARAMETER:: WP = SELECTED_REAL_KIND(10,50)
!  REAL(WP),PARAMETER:: ZERO=0.0, HALF=0.5, ONE=1.0, TWO=2.0
  REAL(WP),PARAMETER:: PI = 3.141592653589793238462643_WP

  REAL(WP),PARAMETER:: GAMMA = 1.4
  INTEGER:: errCode
  CHARACTER(LEN=*),PARAMETER:: FMT='(F5.1,7F8.3)'
  INTEGER:: j,k
  INTEGER,PARAMETER:: NMACH=7, NDELTA=8  
  REAL(WP),DIMENSION(NMACH),PARAMETER:: MACHTAB = &
    (/ 1.5, 2.0, 2.5, 3.0, 4.0, 5.0, 10.0 /)
  REAL(WP),DIMENSION(NDELTA),PARAMETER:: DELTATAB = &
    (PI/180)*(/ 5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0 /)  
  REAL(WP),DIMENSION(NMACH):: thetaTab
! ---------------------------------------------------------------------------
  WRITE(*,FMT) 0.0, MACHTAB
  DO k=1,NDELTA
    DO j=1,NMACH
      CALL ShockAngle(DELTATAB(k),MACHTAB(j),GAMMA, thetaTab(j),errCode)
    END DO
    WRITE(*,FMT) (180/PI)*DELTATAB(k), (180/PI)*thetaTab
  END DO
  
  STOP
END Program ObliqueShockStudy   ! ===========================================

